@@ -17,7 +17,7 @@
 -- {god gift} for god gifts
 --
 -- Item annotations are always prefixed to the item name. For instance:
--- {artefact} the Staff of Wucad Mu
+-- {artefact} the crystal ball of Wucad Mu
 ---------------------------------------------------------------------------
 
 -- Annotate items for searches
@@ -46,13 +46,21 @@ function ch_stash_search_annotate_item(it)
     annot = annot .. "{ego} {branded} "
   end
 
+  if it.is_xp_evoker then
+    annot = annot .. "{evoker} "
+  end
+
   if it.god_gift then
     annot = annot .. "{god gift} "
   end
 
   local skill = it.weap_skill
   if skill then
-    annot = annot .. "{" .. skill .. "} "
+    local skills = crawl.split(skill, ",")
+    for i = 1, #skills, 1
+    do
+        annot = annot .. "{" .. skills[i] .. "} "
+    end
     local hands = it.hands
     local hands_adj
     if hands == 2 then
@@ -83,10 +91,25 @@ function ch_stash_search_annotate_item(it)
       end
       annot = annot .. "} "
     else
+      if it.ego_type_terse == "Fly" then
+        annot = annot .. "{flight} "
+      end
       annot = annot .. "{" .. it.ego_type_terse .. "} "
     end
   end
 
+  if it.class(true) == "potion" or it.class(true) == "scroll" then
+    local props = {
+      ["enlightenment"] = "Will+ flight Fly",
+      ["lignification"] = "rPois rTorment rDrown",
+      ["resistance"] = "rF+ rC+ rElec rPois rCorr",
+      ["revelation"] = "sInv"
+    }
+    if props[it.subtype()] then
+      annot = annot .. "{" .. props[it.subtype()] .. "} "
+    end
+  end
+
   if it.class(true) == "magical staff" and not it.artefact then
     annot = annot .. "{weapon} "
     local props = {
@@ -94,7 +117,7 @@ function ch_stash_search_annotate_item(it)
       ["cold"] = "rC+",
       ["death"] = "rN+",
       ["fire"] = "rF+",
-      ["poison"] = "rPois"
+      ["alchemy"] = "rPois"
     }
     if props[it.subtype()] then
       annot = annot .. "{" .. props[it.subtype()] .. "} "
@@ -113,7 +136,7 @@ function ch_stash_search_annotate_item(it)
       ["pearl"] = "rN+",
       ["storm"] = "rElec",
       ["shadow"] = "Stlth++++",
-      ["gold"] = "rF+ rC+ rPois"
+      ["golden"] = "rF+ rC+ rPois"
     }
     local t = it.name("base"):match("%w+")
     if props[t] then
@@ -162,6 +185,11 @@ function ch_stash_search_annotate_item(it)
     annot = annot .. " {MR} {resist magic} {magic resistance}"
   end
 
+  -- Tag revelation as mapping for back-compat.
+  if it.class(true) == "scroll" and it.subtype() == "revelation" then
+    annot = annot .. " {magic mapping}"
+  end
+
   return annot
 end
 
