@@ -157,7 +157,7 @@ end
 -- @param custom_loot If non-nil, place this as a guaranteed loot item.
 function gauntlet_arena_item_setup(e, custom_loot)
     -- Replace all 'd' after the first with the unused 'X' glyph so we can more
-    -- easilly apply any custom replacement for those extra items.
+    -- easily apply any custom replacement for those extra items.
     e.nsubst("d = d / X")
 
     -- Redefine these item class glyphs to use no_pickup
@@ -364,7 +364,7 @@ tier1_gauntlet_arenas = {
     plant  = "demonic",
   },
   {
-    second = {mons = "shadow", min = 2, max = 4},
+    second = {mons = "shadowghast", min = 2, max = 4},
     liquid = "water",
     plant  = "withered",
   },
@@ -383,10 +383,8 @@ tier1_gauntlet_arenas = {
     liquid = "lava",
   },
   {
-    second = {mons = "shapeshifter", min = 1, max = 3},
-  },
-  {
-    first  = {mons = "flying skull band", min = 1, max = 1},
+    first  = {mons = "laughing skull", min = 1, max = 1},
+    second = {mons = "laughing skull", min = 1, max = 2},
     plant  = "withered",
   },
   {
@@ -407,20 +405,20 @@ tier1_gauntlet_arenas = {
   },
   {
     first  = {mons = "death scarab", min = 1, max = 1},
-    second = {mons = "spectre place:Lair:6", min = 1, max = 1},
+    second = {mons = "spectre place:Lair:$", min = 1, max = 1},
     liquid = "water",
     plant  = "withered",
   },
   {
     first  = {mons = "shining eye", min = 1, max = 1},
-    second = {mons = "floating eye / golden eye", min = 1, max = 2},
+    second = {mons = "glass eye / golden eye", min = 1, max = 2},
     loot   = "potion of mutation ident:type",
     plant  = "demonic",
     weight = 5,
   },
   {
     first  = {mons = "eye of devastation", min = 1, max = 1},
-    second = {mons = "floating eye / golden eye", min = 1, max = 2},
+    second = {mons = "glass eye / golden eye", min = 1, max = 2},
     weight = 5,
   },
   {
@@ -428,12 +426,12 @@ tier1_gauntlet_arenas = {
     second = {mons = "tengu conjurer", min = 1, max = 2},
   },
   {
-    first  = {mons = "wizard", min = 1, max = 1},
+    first  = {mons = "arcanist", min = 1, max = 1},
     second = {mons = "white imp / shadow imp", min = 1, max = 3},
   },
   {
-    second = {mons = "large abomination", min = 0, max = 2},
-    third  = {mons = "small abomination", min = 3, max = 6},
+    second = {mons = "large abomination", min = 0, max = 1},
+    third  = {mons = "small abomination", min = 3, max = 4},
     plant  = "demonic",
   },
   {
@@ -476,7 +474,7 @@ tier2_gauntlet_arenas = {
     liquid = "lava",
   },
   {
-    second = {mons = "shadow", min = 3, max = 6},
+    second = {mons = "shadowghast", min = 3, max = 6},
     liquid = "water",
     plant  = "withered",
   },
@@ -510,7 +508,7 @@ tier2_gauntlet_arenas = {
     plant  = "demonic",
   },
   {
-    first  = {mons = "hell beast", min = 1, max = 1},
+    first  = {mons = "sin beast", min = 1, max = 1},
     second = {mons = "hell hound", min = 0, max = 2},
     liquid = "lava",
     plant  = "demonic",
@@ -554,16 +552,13 @@ tier2_gauntlet_arenas = {
     liquid = "water",
     plant  = "withered",
   },
-  {
-    first  = {mons = "glowing shapeshifter", min = 1, max = 1},
-    second = {mons = "shapeshifter", min = 2, max = 3},
-  },
   {
     first  = {mons = "apocalypse crab", min = 1, max = 1},
     plant  = "demonic",
   },
   {
-    second = {mons = "flying skull band", min = 2, max = 3},
+    first  = {mons = "weeping skull", min = 1, max = 1},
+    second = {mons = "laughing skull", min = 2, max = 4},
     plant  = "withered",
   },
   {
@@ -599,7 +594,7 @@ tier2_gauntlet_arenas = {
   },
   {
     second = {mons = "thrashing horror", min = 1, max = 2},
-    third  = {mons = "small abomination", min = 2, max = 4},
+    third  = {mons = "small abomination", min = 2, max = 3},
     plant  = "demonic",
   },
   {
@@ -613,18 +608,18 @@ tier2_gauntlet_arenas = {
     plant  = "demonic",
   },
   {
-    second = {mons = "large abomination", min = 2, max = 3},
-    third  = {mons = "small abomination", min = 3, max = 6},
+    second = {mons = "large abomination", min = 1, max = 2},
+    third  = {mons = "small abomination", min = 3, max = 5},
     plant  = "demonic",
   },
   {
     first  = {mons = "wretched star", min = 1, max = 1},
-    second = {mons = "neqoxec / small abomination", min = 1, max = 3},
+    second = {mons = "neqoxec / small abomination", min = 1, max = 2},
     loot   = "potion of mutation ident:type",
     plant  = "demonic",
   },
   {
-    second = {mons = "wizard", min = 2, max = 2},
+    second = {mons = "arcanist / occultist", min = 2, max = 2},
     third  = {mons = "white imp / shadow imp", min = 3, max = 6},
   },
   {
@@ -635,7 +630,7 @@ tier2_gauntlet_arenas = {
   },
   {
     second = {mons = "death scarab", min = 2, max = 2},
-    third  = {mons = "spectre place:Lair:6", min = 2, max = 4},
+    third  = {mons = "spectre place:Lair:$", min = 2, max = 4},
     liquid = "water",
     plant  = "withered",
   },
